@@ -13,6 +13,6 @@
 # limitations under the License.
 # ==============================================================================
 """Activations package definition."""
-from tf_transformers.activations.gelu import gelu
+from tf_transformers.activations.gelu import gelu, quick_gelu
 from tf_transformers.activations.swish import hard_swish, identity, simple_swish
 from tf_transformers.activations.utils import get_activation
